package org.bench4q.agent.datacollector.impl;

import java.lang.reflect.Field;

import org.bench4q.agent.scenario.engine.BehaviorResult;
import org.bench4q.share.models.agent.statistics.BehaviorStatusCodeResultModel;

public class BehaviorStatusCodeCollector {
	private int behaviorId;
	private int statusCode;
	private long count;
	private long countThisTime;
	private long contentLength;
	private long minResponseTime;
	private long maxResponseTime;
	private long totalResponseTimeThisTime;
	private String contentType;

	public BehaviorStatusCodeCollector(int behaviorId, int statusCode,
			String contentType) {
		this.behaviorId = behaviorId;
		this.statusCode = statusCode;
		this.contentType = contentType;
		this.count = 0;
		this.contentLength = 0;
		this.minResponseTime = Long.MAX_VALUE;
		this.maxResponseTime = Long.MIN_VALUE;
		resetTempraryField();
	}

	public void resetTempraryField() {
		this.countThisTime = 0;
		this.totalResponseTimeThisTime = 0;
	}

	public static boolean isSuccess(int statusCode) {
		return statusCode == 200;
	}

	public void add(BehaviorResult behaviorResult) {
		if (behaviorResult == null
				|| behaviorResult.getBehaviorId() != this.behaviorId
				|| behaviorResult.getStatusCode() != this.statusCode) {
			return;
		}
		this.count++;
		this.countThisTime++;
		if (behaviorResult.getSuccessCount() == 0) {
			this.contentLength += 0;
			this.totalResponseTimeThisTime = 0;
			return;
		}
		this.contentLength += behaviorResult.getContentLength();
		this.totalResponseTimeThisTime += behaviorResult.getResponseTime();
		if (behaviorResult.getResponseTime() > this.maxResponseTime) {
			this.maxResponseTime = behaviorResult.getResponseTime();
		}
		if (behaviorResult.getResponseTime() < this.minResponseTime) {
			this.minResponseTime = behaviorResult.getResponseTime();
		}
	}

	private BehaviorStatusCodeResultModel buildModel() {
		BehaviorStatusCodeResultModel result = new BehaviorStatusCodeResultModel();
		result.setBehaviorId(this.behaviorId);
		result.setContentLength(this.contentLength);
		result.setContentType(this.contentType);
		result.setCount(this.count);
		result.setCountThisTime(this.countThisTime);
		result.setMaxResponseTime(this.maxResponseTime);
		result.setMinResponseTime(this.minResponseTime);
		result.setStatusCode(this.statusCode);
		result.setTotalResponseTimeThisTime(this.totalResponseTimeThisTime);
		return result;
	}

	public BehaviorStatusCodeResultModel getStatistics() {
		BehaviorStatusCodeResultModel result = buildModel();
		this.resetTempraryField();
		return result;
	}

	public boolean equals(Object expectedObj) {
		Field[] fields = this.getClass().getDeclaredFields();
		boolean equal = true;
		try {
			for (Field field : fields) {
				field.setAccessible(true);
				if (field.getName().equals("contentType")) {
					field.get(expectedObj).equals(field.get(this));
					continue;
				}
				if (field.getLong(this) != field.getLong(expectedObj)) {
					System.out.println(field.getName()
							+ " is diferent, this is " + field.getLong(this)
							+ ", and the expected is "
							+ field.getLong(expectedObj));
					equal = false;
				}

			}
		} catch (Exception e) {
			e.printStackTrace();
			equal = false;

		}
		return equal;
	}
}
