package org.bench4q.agent.scenario.node;

import java.util.ArrayList;
import java.util.Stack;

import org.bench4q.agent.datacollector.DataCollector;
import org.bench4q.agent.scenario.Parameter;
import org.bench4q.agent.scenario.instruction.Instruction;
import org.bench4q.agent.scenario.instruction.TestInstruction;
import org.bench4q.share.models.agent.ParameterModel;
import org.bench4q.share.models.agent.scriptrecord.BehaviorModel;
import org.bench4q.share.models.agent.statistics.BehaviorBriefModel;

/**
 * This class is the base class of those nodes that will be implemented in
 * plugin
 * 
 * This kind of design has some problems that PluginImplementBehavior need not
 * extends Behavior
 * 
 * @author chentienan
 *
 */
public abstract class PluginImplementBehavior extends Behavior {
	private String use;
	private String name;
	private Parameter[] parameters;

	public String getUse() {
		return use;
	}

	public void setUse(String use) {
		this.use = use;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public Parameter[] getParameters() {
		return parameters;
	}

	public void setParameters(Parameter[] parameters) {
		this.parameters = parameters;
	}

	public abstract boolean shouldBeCount();

	public abstract BehaviorBriefModel getBehaviorBriefResult(
			DataCollector dataStatistics);

	public String getSpecificParamValue(String paramName) {
		for (Parameter parameter : this.getParameters()) {
			if (parameter.getKey().equalsIgnoreCase(paramName)) {
				return parameter.getValue();
			}
		}
		return "";
	}

	protected void checkCondition(ArrayList<Instruction> code,
			Stack<TestInstruction> conditions) {
		code.addAll(conditions);
	}

	protected void buildAndGenerateParameterPart(BehaviorModel behaviorModel) {
		this.setName(behaviorModel.getName());
		this.setUse(behaviorModel.getUse());
		this.setId(behaviorModel.getId());
		this.setParameters(new Parameter[behaviorModel.getParameters().size()]);

		for (int k = 0; k < behaviorModel.getParameters().size(); k++) {
			ParameterModel parameterModel = behaviorModel.getParameters()
					.get(k);
			this.getParameters()[k] = Parameter.generateParamParts(
					parameterModel.getKey(), parameterModel.getValue());
		}
	}
}
